// File    : BLDRSC.CPP
// Author  : Eric Woodruff
// Updated : Wed 08/02/95 14:37:14
// Note    : Copyright 1993-95, Eric Woodruff, All rights reserved
// Compiler: Borland C++ 3.1 to 4.xx
//
// This file contains the standard resource file builder.
// Modify or add the necessary items and compile.
//

// NOTE: The 32 bit version of the resource file is *NOT* compatible with
//       the 16 bit version!  This is because 'int' values stored in it are
//       32 bits long not 16 so they no long match 'short' values which are
//       16 bits long in either version.  See HISTORY.DOC for a required fix
//       to TSTATUSL.CPP before building the 32 bit version of the resource
//       file.
//
#if defined(__DPMI16__)
    #define RSC_FILENAME    "MIBXDM16.RSC"   // File to create.
#else
    #if defined(__DPMI32__)
        #define RSC_FILENAME    "MIBXDM32.RSC"   // File to create.
    #else
        #define RSC_FILENAME    "MIBXDEMO.RSC"   // File to create.
    #endif
#endif

#if _TV_VERSION == 0x0103
#define WriteResource(a,b,c) \
    cout << c << endl; \
    a = b(); \
    if( a == 0 ) \
    { \
        strcpy(endString, RSC_FILENAME ": Creation failure\n"); \
        TObject::destroy(rsc); \
        exit(1); \
    } \
    rsc->put(a,c); \
    TObject::destroy(a)

#define WriteObject(a,c) \
    cout << c << endl; \
    if(a == 0) \
    { \
        strcpy(endString, RSC_FILENAME ": Creation failure\n"); \
        TObject::destroy(rsc); \
        exit(1); \
    } \
    rsc->put(a,c); \
    TObject::destroy(a)
#else
#define WriteResource(a,b,c) \
    cout << c << endl; \
    a = b(); \
    if( a == 0 ) \
    { \
        cout << endl << RSC_FILENAME << ": Creation failure" << endl; \
        TObject::destroy(rsc); \
        exit(1); \
    } \
    rsc->put(a,c); \
    TObject::destroy(a)

#define WriteObject(a,c) \
    cout << c << endl; \
    if(a == 0) \
    { \
        cout << endl << RSC_FILENAME << ": Creation failure" << endl; \
        TObject::destroy(rsc); \
        exit(1); \
    } \
    rsc->put(a,c); \
    TObject::destroy(a)
#endif

#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define Uses_TApplication
#define Uses_TButton
#define Uses_TCheckBoxes
#define Uses_TColorDialog
#define Uses_TColorGroup
#define Uses_TColorItem
#define Uses_TDialog
#define Uses_TInputLine
#define Uses_TKeys
#define Uses_TLabel
#define Uses_TMenuBar
#define Uses_TMenuItem
#define Uses_TProgram
#define Uses_TPXPictureValidator   // This is only pertinent when using TV 2.0
#define Uses_TRadioButtons
#define Uses_TRect
#define Uses_TResourceFile
#define Uses_TScreen
#define Uses_TSItem
#define Uses_TStaticText
#define Uses_TStatusDef
#define Uses_TStatusItem
#define Uses_TStatusLine
#define Uses_TStreamableClass
#define Uses_TSubMenu
#define Uses_TView
#define Uses_TVCOLR             // Use this if you modified the TV.H file.
#define Uses_fpstream
#include <tv.h>

#if !defined(cpDefSize)
// Use this if you chose not to modify the Turbo Vision files.
#include <tvcolr.h>
#endif

#define Uses_TExplodeWindow
#define Uses_TExplodeDialog
#include <texplode.h>

#include <msginpbx.h>

#include <tthermo.h>

#include "link.h"
#include "mibxdemo.h"

// ============================================================================
TDialog *Colors(void);
TMenuBar *initMenuBar(void);
TStatusLine *initStatusLine(void);

TDialog *makeTextDialog(void);
TDialog *makeParamDialog(void);
TExplodeWindow *makeRscExpWin(void);

TDialog *createOptionsDlg(void);
TDialog *createTestDlg(void);
Boolean createTWindowDemo(void);

// ============================================================================

fpstream *s;
TResourceFile* rsc;

// ============================================================================

// The exit function is no longer needed in the TV 2.0 version due to
// the new TScreen::clearOnSuspend data member.
#if _TV_VERSION == 0x0103

char endString[255] = "";

void exitfunc(void)
{
   cout << endl << endString << endl;
}

#pragma exit exitfunc 31

#endif

void main(void)
{
    TDialog *dlg;
    TMenuBar *menuBar;
    TStatusLine *statusLine;

    // Return code variable used by the message/input box builder.
    int RetCode;

#if _TV_VERSION != 0x0103
    // Prevent the screen from being cleared on exit.
    TScreen::clearOnSuspend = False;
#endif

    cout << "Creating " RSC_FILENAME << endl;

    // Construct stream and resource.
    s = new fpstream (RSC_FILENAME, ios::trunc | ios::binary);
    rsc = new TResourceFile(s);

    // Create all of the necessary dialogs for the application.
//    dlg = makeMsgDialog("About", 1, mdNoButtons, ECode,
//        mdXTime | mdInfoColor, 4,
//        "\003Message/Input Box Library "    // \003 = Center lines.
//        "Version " MIBXVERSION_STRING "\n"  // \n = Start new line.
//        "\003by Eric Woodruff\n\n"          // \n\n = Make a blank line.
//        "\003CIS ID: 72134,1150");
//    WriteObject(dlg, "AboutBox");

    WriteResource(dlg, Colors, "ColorDlg");

    WriteResource(dlg, makeTextDialog, "TextDialog");
    WriteResource(dlg, makeParamDialog, "ParamDialog");
    WriteResource(dlg, createOptionsDlg, "OptionsDialog");
    WriteResource(dlg, createTestDlg, "BarDemo4");

    if(!createTWindowDemo())        // Writes directly to the stream.
    {
        cout << RSC_FILENAME << ": Creation failure" << endl; \
        TObject::destroy(rsc); \
        exit(1); \
    }

    // Create a message dialog box for writing to the stream.  Note that you
    // cannot pass variable parameters to the message lines in the main
    // application when restoring a message or input box from a stream, but
    // you can when you create it.  Here, the #defined resource filename
    // goes into creating a constant string for the message dialog box.
    dlg = makeMsgDialog("Message Box from a resource file", 3, mdOKOnly,
                        RetCode, mdTimeBeep | mdHelpCtx, 10, mdShortBeep,
                        mdMedPitch, "Had a beep and will close in 10 seconds",
                        "Read from resource file %s", RSC_FILENAME,
                        "Press F1 to see if the help context is 12345", 12345);

    WriteObject(dlg, "StrmMsgBox");

    // It is important that when you create an input box this way, you
    // insure that the message length and optional history id are accurate!
    dlg = makeMsgDialog("Input box from a resource file", 1, mdYesNoCancel,
                        RetCode, mdTimeBeepHis | mdHelpCtx, 10, mdMediumBeep,
                        mdHighPitch, TEST_STR_SIZE, HISTORY_ID,
                        "This had a beep, will close in 10 seconds, and"
                        " has an input line with history.  The data will"
                        " show up in TestString if <Yes> is selected.", 3000);

    WriteObject(dlg, "StrmInpBox");

#if _TV_VERSION == 0x0200

    // This input box is like the one above, but will have a validated
    // input line that capitalizes the first letter of every word.
    TPXPictureValidator *val = new TPXPictureValidator("!*[ * !,@]", False);

    dlg = makeMsgDialog("Validator input box from a resource file", 1,
                        mdYesNoCancel, RetCode, mdTimeBeepHV | mdHelpCtx, 10,
                        mdMediumBeep, mdHighPitch, TEST_STR_SIZE, HISTORY_ID,
                        val, "This had a beep, will close in 10 seconds, and"
                        " has an input line with history and a validator.  The"
                        " data will show up in TestString if <Yes> is"
                        " selected.", 3001);

    WriteObject(dlg, "StrmValBox");

#endif

    TExplodeWindow *expWin = makeRscExpWin();
    WriteObject(expWin, "ExpWinDemo");

    // Create the menus and status line.
    WriteResource(menuBar, initMenuBar, "MenuBar");
    WriteResource(statusLine, initStatusLine, "StatusLine");

    TObject::destroy(rsc);

#if _TV_VERSION == 0x0103
    strcpy(endString, RSC_FILENAME ": Creation completed.");
#else
    cout << endl << RSC_FILENAME << ": Creation completed." << endl;
#endif

    exit(0);
}

// ============================================================================
TDialog *Colors(void)
{
    TColorGroup &group1 =
        *new TColorGroup("Desktop") +
            *new TColorItem("Background",        1)+

        *new TColorGroup("Menus") +
            *new TColorItem("Normal text",       2)+
            *new TColorItem("Disabled text",     3)+
            *new TColorItem("Shortcut key",      4)+
            *new TColorItem("Selection bar",     5)+
            *new TColorItem("Selected disabled", 6)+
            *new TColorItem("Shortcut selected", 7);

    TColorGroup &group2 =
        *new TColorGroup("Blue Window") +
            *new TColorItem("Frame inactive",    8)+
            *new TColorItem("Frame active",      9)+
            *new TColorItem("Frame icons",      10)+
            *new TColorItem("Scrollbar page",   11)+
            *new TColorItem("Scrollbar icons",  12)+
            *new TColorItem("Normal text",      13)+
            *new TColorItem("Selected text",    14)+

        *new TColorGroup("Cyan Window") +
            *new TColorItem("Frame inactive",  16)+
            *new TColorItem("Frame active",    17)+
            *new TColorItem("Frame icons",     18)+
            *new TColorItem("Scrollbar page",  19)+
            *new TColorItem("Scrollbar icons", 20)+
            *new TColorItem("Normal text",     21)+
            *new TColorItem("Selected text",   22)+

        *new TColorGroup("Gray Window") +
            *new TColorItem("Frame inactive",  24)+
            *new TColorItem("Frame active",    25)+
            *new TColorItem("Frame icons",     26)+
            *new TColorItem("Scrollbar page",  27)+
            *new TColorItem("Scrollbar icons", 28)+
            *new TColorItem("Normal text",     29)+
            *new TColorItem("Selected text",   30);

    TColorGroup &group3 =
        *new TColorGroup("Dialog Box") +
            *new TColorItem("Frame inactive",   32)+
            *new TColorItem("Frame active",     33)+
            *new TColorItem("Frame icons",      34)+
            *new TColorItem("Scrollbar page",   35)+
            *new TColorItem("Scrollbar icons",  36)+

            // The last four text colors can be set, but are only
            // accessible when using the alternate mapColor() routine.
            // from TVCOLR.ZIP/TVCLR2.ZIP.
            *new TColorItem("Normal text",       37)+
            *new TColorItem("Information",  cCTxtInfoColor)+    // Extended
            *new TColorItem("Notification", cCTxtNotifyColor)+  // Extended
            *new TColorItem("Warning",      cCTxtWarnColor)+    // Extended
            *new TColorItem("Error",        cCTxtErrorColor)+   // Extended

        // Selected Shortcut is only applicable if you made the
        // changes described in COLUPDT.DOC in TVCOLR.ZIP/TVCLR2.ZIP.
            *new TColorItem("Normal label",            38)+
            *new TColorItem("Selected label",          39)+
            *new TColorItem("Label shortcut key",      40)+
#ifdef MODIFIED_TV_COLORS
    #if _TV_VERSION == 0x0103
            *new TColorItem("Selected shortcut", 66)+   // Added by me - TV 1.03 location
    #else
            *new TColorItem("Selected label shortcut key", 138)+  // Same, but TV 2.0 location
    #endif
#endif

        // Default and Selected Shortcut are only applicable if you made
        // the changes described in COLUPDT.DOC in TVCOLR.ZIP/TVCLR2.ZIP
            *new TColorItem("Normal button",            41)+
            *new TColorItem("Default button",           42)+
            *new TColorItem("Selected button",          43)+
            *new TColorItem("Disabled button",          44)+
            *new TColorItem("Button shortcut key",      45)+
#ifdef MODIFIED_TV_COLORS
    #if _TV_VERSION == 0x0103
            *new TColorItem("Default shortcut",  64)+   // Added by me - TV 1.03 location
            *new TColorItem("Selected shortcut", 65)+
    #else
            *new TColorItem("Default button shortcut key",  136)+ // Same, but TV 2.0 location
            *new TColorItem("Selected button shortcut key", 137)+
    #endif
#endif
            *new TColorItem("Button shadow",            46)+

            *new TColorItem("Cluster normal",    47)+
            *new TColorItem("Cluster selected",  48)+
            *new TColorItem("Cluster shortcut",  49)+

            *new TColorItem("Input line normal",    50)+
            *new TColorItem("Input line selected",  51)+
            *new TColorItem("Input line arrow",     52)+
            *new TColorItem("History button",       53)+
            *new TColorItem("History sides",        54)+
            *new TColorItem("History scrollbar",    55)+
            *new TColorItem("History icons",        56)+

            *new TColorItem("Normal list item",     57)+
            *new TColorItem("Focused list item",    58)+
            *new TColorItem("Selected list item",   59)+
            *new TColorItem("List divider",         60)+
            *new TColorItem("Information pane",     61);

    TColorGroup &group4 =
        *new TColorGroup("Help Window") +
#if _TV_VERSION == 0x0103
    #ifdef MODIFIED_TV_COLORS
            *new TColorItem("Frame inactive",   67)+        // TV 1.03 locations after
            *new TColorItem("Frame active",     68)+        // TVCOLR/TVCLR2.ZIP modifications.
            *new TColorItem("Frame icons",      69)+
            *new TColorItem("Scrollbar page",   70)+
            *new TColorItem("Scrollbar icons",  71)+
            *new TColorItem("Normal text",      72)+
            *new TColorItem("Keyword",          73)+
            *new TColorItem("Selected keyword", 74);
    #else
            *new TColorItem("Frame inactive",   64)+        // Standard unmodified TV 1.03
            *new TColorItem("Frame active",     65)+        // locations.
            *new TColorItem("Frame icons",      66)+
            *new TColorItem("Scrollbar page",   67)+
            *new TColorItem("Scrollbar icons",  68)+
            *new TColorItem("Normal text",      69)+
            *new TColorItem("Keyword",          70)+
            *new TColorItem("Selected keyword", 71);
    #endif
#else
            *new TColorItem("Frame inactive",   128)+       // The help colors are a part
            *new TColorItem("Frame active",     129)+       // of the base palette in
            *new TColorItem("Frame icons",      130)+       // TV 2.0.  These are the standard
            *new TColorItem("Scrollbar page",   131)+       // unmodified locations.
            *new TColorItem("Scrollbar icons",  132)+
            *new TColorItem("Normal text",      133)+
            *new TColorItem("Keyword",          134)+
            *new TColorItem("Selected keyword", 135);
#endif

    TColorGroup &All = group1 + group2 + group3 + group4;

    // See the notes in handleEvent() about the proper usage of TColorDialog.
    TColorDialog *c = new TColorDialog((TPalette*)0, &All);

    return c;
}

TMenuBar *initMenuBar(void)
{
    TSubMenu& MiscMenu = *new TSubMenu( "~\xF0~", 0) +
        *new TMenuItem( "~A~bout...", cmAbout, kbNoKey)+
        *new TMenuItem( "~R~epaint desktop", cmRepaint, kbNoKey) +
            newLine() +
        *new TMenuItem( "~D~OS shell", cmDosShell, kbNoKey) +
        *new TMenuItem( "~Q~uit", cmQuit, kbAltX, hcNoContext, "Alt+X" );

    TSubMenu& MIBXMenu = *new TSubMenu("~M~essage/Input Boxes", 0) +
        *new TMenuItem("~B~utton Menu", cmButtonMenu, kbF2, hcNoContext, "F2")+
        *new TMenuItem("~C~hange Text", cmChgText, kbNoKey, hcNoContext)+
        *new TMenuItem("~T~est Message Box", cmTestMsgBox, kbNoKey, hcNoContext)+
        *new TMenuItem("T~e~st Input Box", cmTestInpBox, kbNoKey, hcNoContext)+
        *new TMenuItem("M~s~gBox on Stream", cmStrmMsgBox, kbNoKey, hcNoContext)+
        *new TMenuItem("~I~npBox on Stream", cmStrmInpBox, kbNoKey, hcNoContext)+
        *new TMenuItem("~F~ormatted Message Test", cmFormatTest, kbNoKey, hcNoContext)+
         newLine()+
        *new TMenuItem("TE~x~plodeWindow demo", cmExpWinDemo, kbF7, hcNoContext, "F7")+
        *new TMenuItem("~G~et TExplodeWindow from stream", cmExpWinRscDemo, kbF8, hcNoContext, "F8");

    TSubMenu& ThermoMenu = *new TSubMenu("~T~hermometer", 0)+
        *new TMenuItem("~T~Dialog demo", cmBarDemo1, kbNoKey, hcNoContext)+
        *new TMenuItem("T~W~indow demo", cmBarDemo2, kbNoKey, hcNoContext)+
        *new TMenuItem("~G~et TWindow from stream", cmBarDemo3, kbNoKey, hcNoContext)+
        *new TMenuItem("G~e~t TDialog from stream", cmBarDemo4, kbNoKey, hcNoContext);

    TSubMenu& WindowMenu = *new TSubMenu( "~W~indows", 0) +
        *new TMenuItem( "~S~ize/Move", cmResize, kbCtrlF5, hcNoContext, "Ctrl+F5" ) +
        *new TMenuItem( "~Z~oom", cmZoom, kbF5, hcNoContext, "F5" ) +
        *new TMenuItem( "C~a~scade", cmCascade, kbNoKey) +
        *new TMenuItem( "~T~ile", cmTile, kbNoKey) +
        *new TMenuItem( "Ne~x~t", cmNext, kbF6, hcNoContext, "F6" ) +
        *new TMenuItem( "~P~revious", cmPrev, kbShiftF6, hcNoContext, "Shift+F6" ) +
        *new TMenuItem( "~C~lose", cmClose, kbAltF3, hcNoContext, "Alt+F3" ) +
        *new TMenuItem( "Clos~e~ all", cmCloseAll, kbNoKey) +
            newLine() +
        *new TMenuItem( "To~g~gle Screen Size", cmScreenSize, kbNoKey) +
        *new TMenuItem( "~U~ser screen", cmUserScreen, kbAltF5, hcNoContext, "Alt+F5");

    TSubMenu& OptionsMenu = *new TSubMenu("~O~ptions", 0) +
        *new TMenuItem( "~C~olors...", cmColors, kbNoKey ) +
        *new TMenuItem( "C~h~ange palettes", cmChangePalettes, kbNoKey ) +
            newLine() +
        *new TMenuItem( "~L~oad configuration...", cmLoadCfg, kbNoKey) +
        *new TMenuItem( "~S~ave configuration...", cmSaveCfg, kbNoKey);

    return new TMenuBar(TRect(0, 0, 80, 1), MiscMenu + MIBXMenu +
        ThermoMenu + WindowMenu + OptionsMenu);
}

TStatusLine *initStatusLine(void)
{
    return new TStatusLine( TRect(0, 24, 80, 25),
      *new TStatusDef( 0, 0xFFFF ) +
        *new TStatusItem( NULL, kbF10, cmMenu ) +
        *new TStatusItem( "~F1~ Help", kbF1, cmHelp ) +
        *new TStatusItem( "~Alt+X~ Exit", kbAltX, cmQuit ) +
        *new TStatusItem( "~F2~ Button Menu", kbF2, cmButtonMenu));
}

// The following two dialog boxes are used for the sole purpose of putting
// the Message Box/Input Box library through its paces.  These are in no way
// required for using the library or functions themselves.
TDialog *makeTextDialog(void)
{
    TExplodeDialog* dlg = new TExplodeDialog(TRect(0,0,80,23),
        "Set Dialog Texts");

    if(!dlg)
        return 0;

    dlg->options |= ofCentered;

    TInputLine *i = new TInputLine(TRect(12,2,43,3), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(2,2,11,3), "Line #1:", i));

    i = new TInputLine(TRect(12,3,43,4), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(2,3,11,4), "Line #2:", i));

    i = new TInputLine(TRect(12,4,43,5), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(2,4,11,5), "Line #3:", i));

    i = new TInputLine(TRect(12,5,43,6), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(2,5,11,6), "Line #4:", i));

    i = new TInputLine(TRect(12,6,43,7), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(2,6,11,7), "Line #5:", i));

    i = new TInputLine(TRect(12,7,43,8), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(2,7,11,8), "Line #6:", i));

    i = new TInputLine(TRect(12,8,43,9), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(2,8,11,9), "Line #7:", i));

    i = new TInputLine(TRect(12,9,43,10), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(2,9,11,10), "Line #8:", i));

    i = new TInputLine(TRect(12,10,43,11), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(2,10,11,11), "Line #9:", i));

    i = new TInputLine(TRect(12,11,43,12), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(1,11,11,12), "Line #10:", i));

    i = new TInputLine(TRect(12,12,43,13), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(1,12,11,13), "Line #11:", i));

    i = new TInputLine(TRect(12,13,43,14), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(1,13,11,14), "Line #12:", i));

    i = new TInputLine(TRect(12,14,43,15), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(1,14,11,15), "Line #13:", i));

    i = new TInputLine(TRect(12,15,43,16), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(1,15,11,16), "Line #14:", i));

    i = new TInputLine(TRect(12,16,43,17), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(1,16,11,17), "Line #15:", i));

    i = new TInputLine(TRect(12,17,43,18), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(1,17,11,18), "Line #16:", i));

    i = new TInputLine(TRect(57,2,78,3), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(45,2,56,3), "Button #1:", i));

    i = new TInputLine(TRect(57,3,78,4), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(45,3,56,4), "Button #2:", i));

    i = new TInputLine(TRect(57,4,78,5), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(45,4,56,5), "Button #3:", i));

    i = new TInputLine(TRect(57,5,78,6), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(45,5,56,6), "Button #4:", i));

    i = new TInputLine(TRect(57,6,78,7), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(45,6,56,7), "Button #5:", i));

    i = new TInputLine(TRect(57,7,78,8), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(45,7,56,8), "Button #6:", i));

    i = new TInputLine(TRect(57,8,78,9), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(45,8,56,9), "Button #7:", i));

    i = new TInputLine(TRect(57,9,78,10), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(45,9,56,10), "Button #8:", i));

    i = new TInputLine(TRect(57,10,78,11), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(45,10,56,11), "Button #9:", i));

    i = new TInputLine(TRect(57,11,78,12), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(44,11,56,12), "Button #10:", i));

    i = new TInputLine(TRect(57,12,78,13), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(44,12,56,13), "Button #11:", i));

    i = new TInputLine(TRect(57,13,78,14), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(44,13,56,14), "Button #12:", i));

    i = new TInputLine(TRect(57,14,78,15), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(44,14,56,15), "Button #13:", i));

    i = new TInputLine(TRect(57,15,78,16), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(44,15,56,16), "Button #14:", i));

    i = new TInputLine(TRect(57,16,78,17), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(44,16,56,17), "Button #15:", i));

    i = new TInputLine(TRect(2,20,43,21), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(1,19,43,20), "Message Dialog Title:", i));

    i = new TInputLine(TRect(63,18,78,19), TEXT_SIZE);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(44,18,62,19), "Validator String:", i));

    TButton *b = new TButton(TRect(46,20,58,22), "O~K~", cmOK, bfDefault);
    dlg->insert(b);

    b = new TButton(TRect(62,20,74,22), "Cancel", cmCancel, bfNormal);
    dlg->insert(b);

    dlg->selectNext(False);
    return dlg;
}

TDialog *makeParamDialog(void)
{
    TDialog* dlg = new TDialog(TRect(0,0,80,23), "Test Parameters");
    if(!dlg)
        return 0;

    dlg->options |= ofCentered;

    TRadioButtons *r = new TRadioButtons(TRect(2,3,16,20),
      new TSItem("No Text",
      new TSItem("1 Line",
      new TSItem("2 Lines",
      new TSItem("3 Lines",
      new TSItem("4 Lines",
      new TSItem("5 Lines",
      new TSItem("6 Lines",
      new TSItem("7 Lines",
      new TSItem("8 Lines",
      new TSItem("9 Lines",
      new TSItem("10 Lines",
      new TSItem("11 Lines",
      new TSItem("12 Lines",
      new TSItem("13 Lines",
      new TSItem("14 Lines",
      new TSItem("15 Lines",
      new TSItem("16 Lines", 0))))))))))))))))));
    dlg->insert(r);

    r = new TRadioButtons(TRect(20,1,45,22),
      new TSItem("No Buttons",
      new TSItem("OK Only",
      new TSItem("Cancel Only",
      new TSItem("OK and Cancel",
      new TSItem("Yes and No",
      new TSItem("Yes, No, and Cancel",
      new TSItem("1 Button",
      new TSItem("2 Buttons",
      new TSItem("3 Buttons",
      new TSItem("4 Buttons",
      new TSItem("5 Buttons",
      new TSItem("6 Buttons",
      new TSItem("7 Buttons",
      new TSItem("8 Buttons",
      new TSItem("9 Buttons",
      new TSItem("10 Buttons",
      new TSItem("11 Buttons",
      new TSItem("12 Buttons",
      new TSItem("13 Buttons",
      new TSItem("14 Buttons",
      new TSItem("15 Buttons", 0))))))))))))))))))))));
    dlg->insert(r);

    TCheckBoxes *x = new TCheckBoxes(TRect(49,1,75,7),
      new TSItem("Will explode/implode",
      new TSItem("Has a History Button",
      new TSItem("Input is a password",
      new TSItem("Has validator (TV 2)",
      new TSItem("Has a help context",
      new TSItem("Disable sounds", 0)))))));
    dlg->insert(x);

    TRadioButtons *y = new TRadioButtons(TRect(49,7,77,12),
      new TSItem("Use normal color",
      new TSItem("Use information color",
      new TSItem("Use notification color",
      new TSItem("Use warning color",
      new TSItem("Use error color", 0))))));
    dlg->insert(y);

    TInputLine *i = new TInputLine(TRect(69,13,76,14), 3);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(47,13,68,14), "Max Input Line Size:", i));

    i = new TInputLine(TRect(69,14,76,15), 5);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(46,14,68,15), "Time Limit (seconds):", i));

    i = new TInputLine(TRect(69,15,76,16), 5);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(48,15,68,16), "Beep (1/4 seconds):", i));

    i = new TInputLine(TRect(69,16,76,17), 5);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(56,16,68,17), "Beep Pitch:", i));

    TButton *b = new TButton(TRect(56,18,70,20), "O~K~", cmOK, bfDefault);
    dlg->insert(b);

    b = new TButton(TRect(56,20,70,22), "~S~top", cmCancel, bfNormal);
    dlg->insert(b);

    dlg->selectNext(False);
    return dlg;
}

TExplodeWindow *makeRscExpWin(void)
{
    TExplodeWindow *xw = new TExplodeWindow(TRect(0,0,80,23),
         "Resource File Exploding Window Demo", 0);

    if(!xw)
        return NULL;

    xw->MakeItExplode(0);           // Explode it as fast as possible.
    xw->options |= ofTileable;
    xw->growMode = gfGrowAll | gfGrowRel;

    TRect r = xw->getExtent();
    r.grow(-1, -1);

    TStaticText *s = new TStaticText(r, "This is a streamed test.");
    s->growMode = gfGrowHiX | gfGrowHiY;
    xw->insert(s);

    return xw;
}

Boolean createTWindowDemo(void)
{
    TWindow *win = new TWindow(TRect(0,1,80,16), "TThermometer Demo", 0);

    if(!win)
        return False;

    win->options = ofCentered;

    TStaticText *s = new TStaticText(TRect(23, 2, 60, 4),
      "Time thermometers (HH:MM:SS.hh)\nPress a key or click mouse to close");

    if(!s)
        return False;

    s->options = ofCenterX;
    win->insert(s);

    s = new TStaticText(TRect(9, 6, 11, 13), "HH\n \nMM\n \nSS\n \nhh");
    if(!s)
        return False;

    win->insert(s);

    // NOTE: Because these are manipulated separately, we must write them
    //       to the resource file separately.

    // 1 block = 1 unit. (0-23 hours, 0-59 minutes/seconds)
    TThermometer *hbar = new TThermometer(TRect(12, 5, 35, 7), NULL, 23, thMsgAbove);
    TThermometer *mbar = new TThermometer(TRect(12, 7, 71, 9), NULL, 59, thMsgAbove);

    // Add a little variety to the bar types.
    TThermometer *sbar = new TThermometer(TRect(12, 9, 71, 11), NULL, 59,
        thMsgAbove, pbfDarkXHatch, pbfCircle1);

    // Can't fit 99 blocks on the screen.  Approximate scale = 2:1.
    TThermometer *tbar = new TThermometer(TRect(12, 11, 62, 13), NULL, 99,
        thMsgAbove, pbfTinyDots, pbfBigDots);

    if(!hbar || !mbar || !sbar || !tbar)
        return False;

    WriteObject(win,  "TWindowDemo");
    WriteObject(hbar, "THourBar");
    WriteObject(mbar, "TMinsBar");
    WriteObject(sbar, "TSecsBar");
    WriteObject(tbar, "THundBar");

    return True;        // Successful.
}

TDialog *createOptionsDlg(void)
{
    char *FillerNames[19] = {
      "Solid", "Empty", "Tiny Dots", "Small Dots", "Big Dots",
      "Circle 1", "Circle 2", "Dashed Line 1", "Dashed Line 2",
      "Dashed Line 3", "Dashed Line 4", "Arrowheads", "Light Crosshatch",
      "Medium Crosshatch", "Dark Crosshatch", "Thin Line", "Double Line",
      "Thick Line 1", "Thick Line 2" };

    TDialog* dlg = new TDialog(TRect(0,0,80,23), "TThermometer Bar Options");
    if (!dlg)
        return NULL;

    TButton *btn = new TButton(TRect(53,20,63,22), "Te~s~t", cmOK, bfDefault);
    dlg->insert(btn);

    btn = new TButton(TRect(67,20,77,22), "Cancel", cmCancel, bfNormal);
    dlg->insert(btn);

    TRadioButtons *r = new TRadioButtons(TRect(3,2,26,21),
      new TSItem(FillerNames[0],
      new TSItem(FillerNames[1],
      new TSItem(FillerNames[2],
      new TSItem(FillerNames[3],
      new TSItem(FillerNames[4],
      new TSItem(FillerNames[5],
      new TSItem(FillerNames[6],
      new TSItem(FillerNames[7],
      new TSItem(FillerNames[8],
      new TSItem(FillerNames[9],
      new TSItem(FillerNames[10],
      new TSItem(FillerNames[11],
      new TSItem(FillerNames[12],
      new TSItem(FillerNames[13],
      new TSItem(FillerNames[14],
      new TSItem(FillerNames[15],
      new TSItem(FillerNames[16],
      new TSItem(FillerNames[17],
      new TSItem(FillerNames[18], 0))))))))))))))))))));
    dlg->insert(r);
    dlg->insert(new TLabel(TRect(2,1,23,2), "~B~ackground Character", r));

    r = new TRadioButtons(TRect(28,2,51,21),
      new TSItem(FillerNames[0],
      new TSItem(FillerNames[1],
      new TSItem(FillerNames[2],
      new TSItem(FillerNames[3],
      new TSItem(FillerNames[4],
      new TSItem(FillerNames[5],
      new TSItem(FillerNames[6],
      new TSItem(FillerNames[7],
      new TSItem(FillerNames[8],
      new TSItem(FillerNames[9],
      new TSItem(FillerNames[10],
      new TSItem(FillerNames[11],
      new TSItem(FillerNames[12],
      new TSItem(FillerNames[13],
      new TSItem(FillerNames[14],
      new TSItem(FillerNames[15],
      new TSItem(FillerNames[16],
      new TSItem(FillerNames[17],
      new TSItem(FillerNames[18], 0))))))))))))))))))));
    dlg->insert(r);
    dlg->insert(new TLabel(TRect(27,1,48,2), "~F~oreground Character", r));

    TCheckBoxes *x = new TCheckBoxes(TRect(53,3,77,9),
        new TSItem("Has a title line",
        new TSItem("Has a message line",
        new TSItem("Has a boxed bar",
        new TSItem("Will close dialog",
        new TSItem("Message above bar",
        new TSItem("Pause half way", 0)))))));
    dlg->insert(x);
    dlg->insert(new TLabel(TRect(52,2,72,3), "~T~hermometer Options", x));

    TInputLine *i = new TInputLine(TRect(53,10,77,11), 256);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(52,9,60,10), "T~i~tle", i));

    i = new TInputLine(TRect(53,12,77,13), 256);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(52,11,60,12), "~M~essage", i));

    i = new TInputLine(TRect(53,15,63,16), 3);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(52,14,63,15), "~W~idth", i));

    i = new TInputLine(TRect(53,18,63,19), 9);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(52,17,63,18), "It~e~rations", i));

    i = new TInputLine(TRect(66,18,74,19), 4);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(65,17,71,18), "~D~elay", i));

    dlg->selectNext(False);
    return dlg;
}

TDialog *createTestDlg(void)
{
    TDialog* dlg = new TDialog(TRect(0,0,61,18), "Demo TThermometer Bar");
    if(!dlg)
        return NULL;

    dlg->options |= ofCentered;

    TStaticText *s = new TStaticText(TRect(4,9,58,10),
        "When running, press a key or click mouse to stop.");
    dlg->insert(s);

    TInputLine *i = new TInputLine(TRect(4,12,58,13), 256);
    dlg->insert(i);
    dlg->insert(new TLabel(TRect(4,11,58,12), "~F~ilespec:", i));

    // NOTE: YOU MUST SET THE bfBroadcast BIT.  OTHERWISE, THE THERMOMETER
    //       WON'T SEE THE BUTTON PRESS!
    TButton *btn = new TButton(TRect(12,14,28,16), "~S~tart Demo", cmOK,
        bfDefault | bfBroadcast);
    dlg->insert(btn);

    btn = new TButton(TRect(38,14,48,16), "~C~ancel", cmCancel, bfNormal);
    dlg->insert(btn);

    TThermometer *bar = new TThermometer(TRect(1, 2, 58, 7), NULL, 0L, thTtlMsgBox);

    bar->options |= ofCenterX;           // Center bar horizontally.
    bar->setTitle("File Search Demo");   // Centered title.
    bar->setMessage("");

    dlg->insert(bar);

    dlg->selectNext(False);     // Always move off of the bar if it was
                                // inserted last.
    return dlg;
}
